#ifndef __DBUS_PROXY_DBUS_EXAMPLE_CALCULATOR_H
#define __DBUS_PROXY_DBUS_EXAMPLE_CALCULATOR_H

#include <dbus-cxx.h>

#include "calculator.h"

#include "calculator_interface.h"

namespace DBus {
  namespace Example {

    class CalculatorProxy : public ::DBus::ObjectProxy, public ::Examples::CalculatorInterface
    {
      protected:
        CalculatorProxy( ::DBus::Connection::pointer conn, const std::string& dest="dbuscxx.example.calculator.server", const std::string& path="/dbuscxx/example/Calculator"):
          ::DBus::ObjectProxy(conn, dest, path)
        {
          ::DBus::Path child_path;

          m_method_add_iii = this->create_method< double,double,double >( "Calculator.Basic", "add" );
          m_method_sub_iii = this->create_method< double,double,double >( "Calculator.Basic", "sub" );
          m_method_mul_iii = this->create_method< double,double,double >( "Calculator.Basic", "mul" );
          m_method_div_iii = this->create_method< double,double,double >( "Calculator.Basic", "div" );
          m_method_pi_i = this->create_method< double >( "Calculator.Basic", "pi" );
          m_method_print_pi_v = this->create_method< void >( "Calculator.Basic", "print_pi" );
          m_method_name_i = this->create_method< std::string >( "Calculator.Basic", "name" );
          m_method_set_name_vi = this->create_method< void,const std::string& >( "Calculator.Basic", "set_name" );
          m_signal_calculation = this->create_signal<void,std::string,std::string,double,double,double>( "Calculator.Basic", "calculation" );
          m_method_factorial_ii = this->create_method< uint64_t,uint8_t >( "Calculator.Computed", "factorial" );
          m_method_fibonacci_ii = this->create_method< uint64_t,uint8_t >( "Calculator.Computed", "fibonacci" );
          m_method_thue_morse_ii = this->create_method< uint64_t,uint8_t >( "Calculator.Computed", "thue_morse" );
          m_signal_computation = this->create_signal<void,std::string,uint64_t,uint8_t>( "Calculator.Computed", "computation" );
        }

      public:

        typedef DBusCxxPointer<CalculatorProxy> pointer;

        static pointer create( ::DBus::Connection::pointer conn,
                               const std::string& dest="dbuscxx.example.calculator.server",
                               const std::string& path="/dbuscxx/example/Calculator"
                             )
        { return pointer( new CalculatorProxy(conn, dest, path)); }


        virtual double add( double a, double b ) const { return (*m_method_add_iii)( a, b); }

        virtual double subtract( double a, double b ) const { return (*m_method_sub_iii)( a, b); }

        virtual double multiply( double a, double b ) const { return (*m_method_mul_iii)( a, b); }

        virtual double divide( double a, double b ) const { return (*m_method_div_iii)( a, b); }

        virtual double pi( ) const { return (*m_method_pi_i)(); }

        virtual void print_pi( ) const { return (*m_method_print_pi_v)(); }

        virtual std::string name( ) const { return (*m_method_name_i)(); }

        virtual void set_name( const std::string& name ) { return (*m_method_set_name_vi)( name); }

        ::DBus::signal_proxy<void,std::string,std::string,double,double,double >& signal_calculation() { return *m_signal_calculation; }

        virtual uint64_t factorial( uint8_t n ) const { return (*m_method_factorial_ii)( n); }

        virtual uint64_t fibonacci( uint8_t n ) const { return (*m_method_fibonacci_ii)( n); }

        virtual uint64_t thue_morse( uint8_t n ) const { return (*m_method_thue_morse_ii)( n); }

        ::DBus::signal_proxy<void,std::string,uint64_t,uint8_t >& signal_computation() { return *m_signal_computation; }


      protected:


        ::DBus::MethodProxy<double,double,double>::pointer m_method_add_iii;
        ::DBus::MethodProxy<double,double,double>::pointer m_method_sub_iii;
        ::DBus::MethodProxy<double,double,double>::pointer m_method_mul_iii;
        ::DBus::MethodProxy<double,double,double>::pointer m_method_div_iii;
        ::DBus::MethodProxy<double>::pointer m_method_pi_i;
        ::DBus::MethodProxy<void>::pointer m_method_print_pi_v;
        ::DBus::MethodProxy<std::string>::pointer m_method_name_i;
        ::DBus::MethodProxy<void,const std::string&>::pointer m_method_set_name_vi;
        ::DBus::signal_proxy<void,std::string,std::string,double,double,double>::pointer m_signal_calculation;

        ::DBus::MethodProxy<uint64_t,uint8_t>::pointer m_method_factorial_ii;
        ::DBus::MethodProxy<uint64_t,uint8_t>::pointer m_method_fibonacci_ii;
        ::DBus::MethodProxy<uint64_t,uint8_t>::pointer m_method_thue_morse_ii;
        ::DBus::signal_proxy<void,std::string,uint64_t,uint8_t>::pointer m_signal_computation;


    };

  }
}

#endif
